import java.awt.EventQueue;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.ConnectException;
import java.net.Socket;
import java.net.UnknownHostException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.text.DefaultCaret;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


public class ChatClientApp {
	private MyClientSocket myThread;
	private JFrame frame;
	private JTextField hostTextField;
	private JTextField textField;
	private Socket socket; 
	private ChatClientPrefs chatClientPrefs;
	private JTextArea textArea;
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					ChatClientApp window = new ChatClientApp();
					window.frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public ChatClientApp() {
		initialize();
		initializeHost();
		initializeSocket();
	}
	private String StackTraceAsString(Exception e) {
		StringWriter sw = new StringWriter();
		e.printStackTrace(new PrintWriter(sw));
		return sw.toString();
	}
	private void initializeSocket() {
		try {
			socket = new Socket(chatClientPrefs.getHost(),8800);
			myThread=new MyClientSocket(socket, textArea);
			myThread.start();
		} catch (ConnectException e) {
			textArea.append("Connection Refused: "+chatClientPrefs.getHost()+"\n");		
		} catch (UnknownHostException e) {
			textArea.append("Unknown Host: "+chatClientPrefs.getHost()+"\n");
			textArea.append(StackTraceAsString(e));
		} catch (IOException e) {
			textArea.append(StackTraceAsString(e));
		}		
	}
	private void initializeHost() {
		try {
			FileInputStream fin = new FileInputStream("ClientMessengerPreferences.ser");
			ObjectInputStream ois = new ObjectInputStream(fin);
			chatClientPrefs = (ChatClientPrefs) ois.readObject();
			ois.close();
		} catch(Exception ex) {
			chatClientPrefs=new ChatClientPrefs();
		} 
		HostDialog ghd=new HostDialog();
		chatClientPrefs.setHost(ghd.getHost(chatClientPrefs.getHost()));
		hostTextField.setText(chatClientPrefs.getHost());
		try {
			FileOutputStream fout = new FileOutputStream("ClientMessengerPreferences.ser");
			ObjectOutputStream oos = new ObjectOutputStream(fout);
			oos.writeObject(chatClientPrefs);	
			oos.close();
		} catch(Exception ex) {
		} 	
	}
	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		frame = new JFrame();
		frame.setBounds(100, 100, 450, 300);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[]{0, 0, 0, 0};
		gridBagLayout.rowHeights = new int[]{0, 0, 0};
		gridBagLayout.columnWeights = new double[]{0.0, 1.0, 1.0, Double.MIN_VALUE};
		gridBagLayout.rowWeights = new double[]{0.0, 1.0, Double.MIN_VALUE};
		frame.getContentPane().setLayout(gridBagLayout);
		
		JLabel lblHost = new JLabel("Host:");
		GridBagConstraints gbc_lblHost = new GridBagConstraints();
		gbc_lblHost.insets = new Insets(0, 0, 5, 5);
		gbc_lblHost.anchor = GridBagConstraints.EAST;
		gbc_lblHost.gridx = 0;
		gbc_lblHost.gridy = 0;
		frame.getContentPane().add(lblHost, gbc_lblHost);
		
		hostTextField = new JTextField();
		hostTextField.setEditable(false);
		GridBagConstraints gbc_hostTextField = new GridBagConstraints();
		gbc_hostTextField.gridwidth = 2;
		gbc_hostTextField.insets = new Insets(0, 0, 5, 5);
		gbc_hostTextField.fill = GridBagConstraints.HORIZONTAL;
		gbc_hostTextField.gridx = 1;
		gbc_hostTextField.gridy = 0;
		frame.getContentPane().add(hostTextField, gbc_hostTextField);
		hostTextField.setColumns(10);
		
		JSplitPane splitPane = new JSplitPane();
		GridBagConstraints gbc_splitPane = new GridBagConstraints();
		gbc_splitPane.gridwidth = 3;
		gbc_splitPane.insets = new Insets(0, 0, 0, 5);
		gbc_splitPane.fill = GridBagConstraints.BOTH;
		gbc_splitPane.gridx = 0;
		gbc_splitPane.gridy = 1;
		frame.getContentPane().add(splitPane, gbc_splitPane);
		
		JPanel panel_1 = new JPanel();
		splitPane.setRightComponent(panel_1);
		GridBagLayout gbl_panel_1 = new GridBagLayout();
		gbl_panel_1.columnWidths = new int[]{0, 0, 0};
		gbl_panel_1.rowHeights = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0};
		gbl_panel_1.columnWeights = new double[]{1.0, 0.0, Double.MIN_VALUE};
		gbl_panel_1.rowWeights = new double[]{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, Double.MIN_VALUE};
		panel_1.setLayout(gbl_panel_1);
		
		textArea = new JTextArea(5, 20);
		textArea.setEditable(false);
		DefaultCaret caret = (DefaultCaret)textArea.getCaret();
		caret.setUpdatePolicy(DefaultCaret.ALWAYS_UPDATE);
		JScrollPane scrollPane = new JScrollPane(textArea);
		GridBagConstraints gbc_scrollPane = new GridBagConstraints();
		gbc_scrollPane.gridheight = 7;
		gbc_scrollPane.gridwidth = 2;
		gbc_scrollPane.insets = new Insets(0, 0, 5, 5);
		gbc_scrollPane.fill = GridBagConstraints.BOTH;
		gbc_scrollPane.gridx = 0;
		gbc_scrollPane.gridy = 0;
		panel_1.add(scrollPane, gbc_scrollPane);
		
		textField = new JTextField();
		GridBagConstraints gbc_textField = new GridBagConstraints();
		gbc_textField.insets = new Insets(0, 0, 0, 5);
		gbc_textField.fill = GridBagConstraints.HORIZONTAL;
		gbc_textField.gridx = 0;
		gbc_textField.gridy = 7;
		panel_1.add(textField, gbc_textField);
		textField.setColumns(10);
		
		JButton btnSend = new JButton("Send");
		btnSend.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String message=textField.getText();
				myThread.send(message);
				textField.setText("");
			}
		});
		GridBagConstraints gbc_btnSend = new GridBagConstraints();
		gbc_btnSend.gridx = 1;
		gbc_btnSend.gridy = 7;
		panel_1.add(btnSend, gbc_btnSend);
		
		JButton btnQuit = new JButton("Quit");
		btnQuit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (myThread!=null) myThread.close();
				System.exit(0);
			}
		});
		splitPane.setLeftComponent(btnQuit);
	}
}
