import java.awt.EventQueue;
import javax.swing.JFrame;
import java.awt.GridBagLayout;
import javax.swing.JLabel;
import java.awt.GridBagConstraints;
import javax.swing.JSplitPane;
import java.awt.Insets;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.ServerSocket;
import java.net.Socket;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JButton;
import javax.swing.text.DefaultCaret;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class ChatServerApp {
	private DefaultListModel listModel;
	private JLabel lblConnections;
	private JTextArea textArea;
	private JFrame frame;
	private JList socketList;
	private Socket clientSocket; 
	private ServerSocket serverSocket=null;
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					ChatServerApp window = new ChatServerApp();
					window.frame.setVisible(true);

				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}
	public ChatServerApp() {initialize();}
	private void initialize() {
		int port = 8800; 
		frame = new JFrame();
		frame.setBounds(100, 100, 450, 300);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[]{0, 0, 0, 0, 0};
		gridBagLayout.rowHeights = new int[]{0, 0, 0};
		gridBagLayout.columnWeights = new double[]{1.0, 1.0, 1.0, 1.0, Double.MIN_VALUE};
		gridBagLayout.rowWeights = new double[]{0.0, 1.0, Double.MIN_VALUE};
		frame.getContentPane().setLayout(gridBagLayout);		
		JLabel lblHostIp = new JLabel("Host IP: 127.0.0.1");
		GridBagConstraints gbc_lblHostIp = new GridBagConstraints();
		gbc_lblHostIp.insets = new Insets(0, 0, 5, 5);
		gbc_lblHostIp.fill = GridBagConstraints.HORIZONTAL;
		gbc_lblHostIp.gridx = 0;
		gbc_lblHostIp.gridy = 0;
		frame.getContentPane().add(lblHostIp, gbc_lblHostIp);
		lblConnections = new JLabel("Connections: 0");
		GridBagConstraints gbc_lblConnections = new GridBagConstraints();
		gbc_lblConnections.insets = new Insets(0, 0, 5, 5);
		gbc_lblConnections.gridx = 1;
		gbc_lblConnections.gridy = 0;
		frame.getContentPane().add(lblConnections, gbc_lblConnections);
		JButton btnRefresh = new JButton("Refresh");
		btnRefresh.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateConnections();
			}
		});
		GridBagConstraints gbc_btnRefresh = new GridBagConstraints();
		gbc_btnRefresh.insets = new Insets(0, 0, 5, 5);
		gbc_btnRefresh.gridx = 2;
		gbc_btnRefresh.gridy = 0;
		frame.getContentPane().add(btnRefresh, gbc_btnRefresh);

		JButton btnDrop = new JButton("Drop");
		btnDrop.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dropSelectedSockets();
			}
		});
		GridBagConstraints gbc_btnDrop = new GridBagConstraints();
		gbc_btnDrop.insets = new Insets(0, 0, 5, 0);
		gbc_btnDrop.gridx = 3;
		gbc_btnDrop.gridy = 0;
		frame.getContentPane().add(btnDrop, gbc_btnDrop);

		JSplitPane splitPane = new JSplitPane();
		GridBagConstraints gbc_splitPane = new GridBagConstraints();
		gbc_splitPane.gridwidth = 4;
		gbc_splitPane.fill = GridBagConstraints.BOTH;
		gbc_splitPane.gridx = 0;
		gbc_splitPane.gridy = 1;
		frame.getContentPane().add(splitPane, gbc_splitPane);

		JScrollPane leftScrollPane = new JScrollPane();
		splitPane.setLeftComponent(leftScrollPane);

		listModel = new DefaultListModel();
		socketList = new JList(listModel);
		leftScrollPane.setViewportView(socketList);

		JScrollPane rightScrollPane = new JScrollPane();
		splitPane.setRightComponent(rightScrollPane);

		textArea = new JTextArea();
		rightScrollPane.setViewportView(textArea);
		DefaultCaret caret = (DefaultCaret)textArea.getCaret();
		caret.setUpdatePolicy(DefaultCaret.ALWAYS_UPDATE);

		try {
			String myIP=TCP_IP_Lib.getMyIP();
			serverSocket = new ServerSocket(port);
			lblHostIp.setText("Host IP:"+myIP);
			new Thread()
			{
				public void run() {
					for(;;) { 
						try {
							clientSocket=serverSocket.accept();
							new Thread(new MyServerSocket(clientSocket, textArea)).start();
							updateConnections();
						} catch (IOException e) {
							textArea.append("Exception connecting...\n");
							textArea.append(StackTraceAsString(e));
						}
					}
				}
			}.start();
		} catch (IOException e) {
			textArea.append("Cannot open Server Socket!\n");
			textArea.append(StackTraceAsString(e));
		}
	}
	private void dropSelectedSockets() {
		int[] sockIndexes=socketList.getSelectedIndices();
		String[] socks=MyServerSocket.getSockets();
		textArea.append("Dropping:");
		for (int i=MyServerSocket.getNumSockets()-1; i>=0; i--) {
			for (int sockIndex : sockIndexes) {
				if (sockIndex==i) {
					textArea.append(socks[sockIndex]+"\n");
					MyServerSocket.closeSocketAtIndex(sockIndex);
				}
			}
		}
		updateConnections();
	}
	private void updateConnections() {
		lblConnections.setText(""+MyServerSocket.getNumSockets());
		listModel.clear();
		String[] socks=MyServerSocket.getSockets();
		for (String aSock : socks) {
			listModel.addElement(aSock);
		}
	}
	private String StackTraceAsString(Exception e) {
		StringWriter sw = new StringWriter();
		e.printStackTrace(new PrintWriter(sw));
		return sw.toString();
	}
}
