import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

public class Input {
	private InputStreamReader rdr; 
	private BufferedReader console; 
	private boolean ready = false;
	private boolean suppressErrorReporting;
	public static String inString() {
		Input input = new Input(true);
		return input.getString();
	}
	public static String inString(String prompt) {
		Input input = new Input(true);
		return input.getString(prompt);		
	}
	public static int inInt() {
		Input input = new Input(true);
		return input.getInt();
	}
	public static int inInt(String prompt) {
		Input input = new Input(true);
		return input.getInt(prompt);		
	}
	public static int inInt(int defaultValue) {
		Input input = new Input(true);
		return input.getInt(defaultValue);					
	}
	public static int inInt(String prompt, int defaultValue) {
		Input input = new Input(true);
		return input.getInt(prompt, defaultValue);			
	}
	public static double inDouble() {
		Input input = new Input(true);
		return input.getDouble();
	}
	public static double inDouble(String prompt) {
		Input input = new Input(true);
		return input.getDouble(prompt);		
	}
	public static double inDouble(double defaultValue) {
		Input input = new Input(true);
		return input.getDouble(defaultValue);					
	}
	public static double inDouble(String prompt, double defaultValue) {
		Input input = new Input(true);
		return input.getDouble(prompt, defaultValue);			
	}
	public Input() {
		suppressErrorReporting = false;
		try {
			rdr = new InputStreamReader(System.in); 
			console = new BufferedReader(rdr);
			ready = true;
		} catch (IllegalArgumentException e) {
			System.out.println("Stop arguing with ME!");
		} catch (Exception e) {
			e.printStackTrace();			
		} catch (Error e) {
			e.printStackTrace();
		}
	}
	public Input(boolean suppressErrorReports) {
		suppressErrorReporting = suppressErrorReports;
		try {
			rdr = new InputStreamReader(System.in); 
			console = new BufferedReader(rdr);
			ready = true;
		} catch (IllegalArgumentException e) {
			System.out.println("Stop arguing with ME!");
		} catch (Exception e) {
			e.printStackTrace();			
		} catch (Error e) {
			e.printStackTrace();
		}
	}
	public void setErrorReporting(boolean suppressErrorReports) {
		suppressErrorReporting = suppressErrorReports;
	}
	public String getString() {
		String result = "";
		if (ready) {
			try {
				result = console.readLine();
			} catch (IOException e) {
				if (!suppressErrorReporting) {
					e.printStackTrace();
				}
			} catch(IllegalArgumentException e) {
				if (!suppressErrorReporting) {
					e.printStackTrace();
				}				
			}
		}
		return result;	
	}
	public String getString(String prompt) {
		System.out.print(prompt);
		return getString();
	}
	public double getDouble(double defaultValue) {
		String inString = getString();
		double result;
		try {
			result = Double.parseDouble(inString);
		} catch(NumberFormatException e) {
			if (!suppressErrorReporting) {
				System.out.println("NumberFormatException: Cannot convert " + inString + " to a Double using parseDouble");
			}
			result = defaultValue;
		}
		return result;		
	}
	public double getDouble(String prompt, double defaultValue) {
		System.out.print(prompt);
		return getDouble(defaultValue);		
	}
	public double getDouble(String prompt) {
		return getDouble(prompt, 0);
	}
	public double getDouble() {
		return getDouble(0);
	}
	public int getInt(int defaultValue) {
		int result;
		String inString = getString();
		try {
			result = Integer.parseInt(inString);
		} catch(NumberFormatException e) {
			if (!suppressErrorReporting) {
				System.out.println("NumberFormatException: Cannot convert " + inString + " to an Integer using parseInt");
			}
			result = defaultValue;
		}
		return result;
	}
	public int getInt(String prompt, int defaultValue) {
		System.out.print(prompt);
		return getInt(defaultValue);
	}
	public int getInt(String prompt) {
		return getInt(prompt, 0);
	}
	public int getInt() {
		return getInt(0);
	}
}
