import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JTextArea;


public class MyServerSocket implements Runnable {
	private static List<MyServerSocket> sockets;
	private JTextArea textArea=null;
	private final Socket clientSocket;
	private PrintStream out;
	private BufferedReader in;
	private boolean alive=false;
	public static int getNumSockets() {
		return sockets.size();
	}
	public static void closeSocketAtIndex(int i) {
		if (i>=0 && i<getNumSockets()) {
			sockets.get(i).closeSocket();
		}
	}
	public String getMyIP() {
		String ip=clientSocket.getRemoteSocketAddress().toString();
		if (ip==null || ip.length()==0) {
			ip="?";
		}
		int slashIndex=ip.indexOf("/");
		if (slashIndex==-1) {
			slashIndex=0;
		}
		int colonIndex=ip.indexOf(":");
		if (colonIndex==-1) colonIndex=ip.length();
		ip=ip.substring(slashIndex+1,colonIndex);
		return ip;
	}
	public static String[] getSockets() {
		String[] socks=new String[sockets.size()];
		for (int i=0; i<sockets.size(); i++) {
			MyServerSocket aChatSocket=sockets.get(i);
			socks[i]=aChatSocket.getMyIP();
		}
		return socks;
	}
	public MyServerSocket(Socket clientSocket) {
		this.clientSocket=clientSocket;
		init(clientSocket);
	}
	public MyServerSocket(Socket clientSocket, JTextArea textArea) {
		this.clientSocket=clientSocket;
		this.textArea=textArea;
		init(clientSocket);
	}
	private void init(Socket clientSocket) {
		if (sockets==null) {
			sockets=new ArrayList<MyServerSocket>();
		}
		try {
			out=new PrintStream(clientSocket.getOutputStream()); 
			InputStreamReader isr=new InputStreamReader(clientSocket.getInputStream()); 
			in=new BufferedReader(isr);
		} catch (IOException e) {
			e.printStackTrace();
		}
		sockets.add(this);
		println("Total Sockets: "+sockets.size());
		alive=true;
	}
	public boolean isAlive() {return alive;}
	public void sendMessage(String message) {
		if (alive) {
			out.println(message);
			out.flush();
			if (out.checkError()) {
				alive=false;
			}
		}
	}
	public void run() {
		while (alive) {
			try {
				String message = in.readLine();
				if (message!=null && message.length()>0) {
					println(getMyIP()+":\""+message+"\"");
					for (int i=0; i<sockets.size(); i++) {
						MyServerSocket aSocket = sockets.get(i);
						aSocket.sendMessage(getMyIP()+":"+message);
					}
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		try {
			clientSocket.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		sockets.remove(this);
	}
	public void closeSocket() {
		alive=false;
	}
	private void println(String s) {
		if (textArea!=null) {
			textArea.append(s+"\n");
		} else {
			System.out.println(s);
		}

	}
}
